@@ -1,5 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2022  Vladimir Golovnev <glassez@yandex.ru>
  * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -28,6 +29,8 @@
 
 #include "mainwindow.h"
 
+#include <QtGlobal>
+
 #include <algorithm>
 #include <chrono>
 
@@ -39,11 +42,13 @@
 #include <QActionGroup>
 #include <QClipboard>
 #include <QCloseEvent>
+#include <QComboBox>
 #include <QDebug>
 #include <QDesktopServices>
 #include <QFileDialog>
 #include <QFileSystemWatcher>
 #include <QKeyEvent>
+#include <QLabel>
 #include <QMessageBox>
 #include <QMetaObject>
 #include <QMimeData>
@@ -107,7 +112,7 @@ using namespace std::chrono_literals;
 namespace
 {
 #define SETTINGS_KEY(name) u"GUI/" name
-#define EXECUTIONLOG_SETTINGS_KEY(name) (SETTINGS_KEY(u"Log/"_qs) name)
+#define EXECUTIONLOG_SETTINGS_KEY(name) (SETTINGS_KEY(u"Log/"_s) name)
 
     const std::chrono::seconds PREVENT_SUSPEND_INTERVAL {60};
 
@@ -120,12 +125,12 @@ namespace
     }
 }
 
-MainWindow::MainWindow(IGUIApplication *app, const State initialState)
+MainWindow::MainWindow(IGUIApplication *app, WindowState initialState)
     : GUIApplicationComponent(app)
     , m_ui(new Ui::MainWindow)
-    , m_storeExecutionLogEnabled(EXECUTIONLOG_SETTINGS_KEY(u"Enabled"_qs))
-    , m_storeDownloadTrackerFavicon(SETTINGS_KEY(u"DownloadTrackerFavicon"_qs))
-    , m_storeExecutionLogTypes(EXECUTIONLOG_SETTINGS_KEY(u"Types"_qs), Log::MsgType::ALL)
+    , m_storeExecutionLogEnabled(EXECUTIONLOG_SETTINGS_KEY(u"Enabled"_s))
+    , m_storeDownloadTrackerFavicon(SETTINGS_KEY(u"DownloadTrackerFavicon"_s))
+    , m_storeExecutionLogTypes(EXECUTIONLOG_SETTINGS_KEY(u"Types"_s), Log::MsgType::ALL)
 {
     m_ui->setupUi(this);
 
@@ -135,7 +140,7 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     m_displaySpeedInTitle = pref->speedInTitleBar();
     // Setting icons
 #ifndef Q_OS_MACOS
-    setWindowIcon(UIThemeManager::instance()->getIcon(u"qbittorrent"_qs));
+    setWindowIcon(UIThemeManager::instance()->getIcon(u"qbittorrent"_s));
 #endif // Q_OS_MACOS
 
 #if (defined(Q_OS_UNIX))
@@ -144,30 +149,30 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
 
     addToolbarContextMenu();
 
-    m_ui->actionOpen->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_qs));
-    m_ui->actionDownloadFromURL->setIcon(UIThemeManager::instance()->getIcon(u"insert-link"_qs));
-    m_ui->actionSetGlobalSpeedLimits->setIcon(UIThemeManager::instance()->getIcon(u"speedometer"_qs));
-    m_ui->actionCreateTorrent->setIcon(UIThemeManager::instance()->getIcon(u"torrent-creator"_qs, u"document-edit"_qs));
-    m_ui->actionAbout->setIcon(UIThemeManager::instance()->getIcon(u"help-about"_qs));
-    m_ui->actionStatistics->setIcon(UIThemeManager::instance()->getIcon(u"view-statistics"_qs));
-    m_ui->actionTopQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-top"_qs));
-    m_ui->actionIncreaseQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-up"_qs));
-    m_ui->actionDecreaseQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-down"_qs));
-    m_ui->actionBottomQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-bottom"_qs));
-    m_ui->actionDelete->setIcon(UIThemeManager::instance()->getIcon(u"list-remove"_qs));
-    m_ui->actionDocumentation->setIcon(UIThemeManager::instance()->getIcon(u"help-contents"_qs));
-    m_ui->actionDonateMoney->setIcon(UIThemeManager::instance()->getIcon(u"wallet-open"_qs));
-    m_ui->actionExit->setIcon(UIThemeManager::instance()->getIcon(u"application-exit"_qs));
-    m_ui->actionLock->setIcon(UIThemeManager::instance()->getIcon(u"object-locked"_qs));
-    m_ui->actionOptions->setIcon(UIThemeManager::instance()->getIcon(u"configure"_qs, u"preferences-system"_qs));
-    m_ui->actionPause->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_qs, u"media-playback-pause"_qs));
-    m_ui->actionPauseAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_qs, u"media-playback-pause"_qs));
-    m_ui->actionStart->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_qs, u"media-playback-start"_qs));
-    m_ui->actionStartAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_qs, u"media-playback-start"_qs));
-    m_ui->menuAutoShutdownOnDownloadsCompletion->setIcon(UIThemeManager::instance()->getIcon(u"task-complete"_qs, u"application-exit"_qs));
-    m_ui->actionManageCookies->setIcon(UIThemeManager::instance()->getIcon(u"browser-cookies"_qs, u"preferences-web-browser-cookies"_qs));
-    m_ui->menuLog->setIcon(UIThemeManager::instance()->getIcon(u"help-contents"_qs));
-    m_ui->actionCheckForUpdates->setIcon(UIThemeManager::instance()->getIcon(u"view-refresh"_qs));
+    m_ui->actionOpen->setIcon(UIThemeManager::instance()->getIcon(u"list-add"_s));
+    m_ui->actionDownloadFromURL->setIcon(UIThemeManager::instance()->getIcon(u"insert-link"_s));
+    m_ui->actionSetGlobalSpeedLimits->setIcon(UIThemeManager::instance()->getIcon(u"speedometer"_s));
+    m_ui->actionCreateTorrent->setIcon(UIThemeManager::instance()->getIcon(u"torrent-creator"_s, u"document-edit"_s));
+    m_ui->actionAbout->setIcon(UIThemeManager::instance()->getIcon(u"help-about"_s));
+    m_ui->actionStatistics->setIcon(UIThemeManager::instance()->getIcon(u"view-statistics"_s));
+    m_ui->actionTopQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-top"_s));
+    m_ui->actionIncreaseQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-up"_s));
+    m_ui->actionDecreaseQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-down"_s));
+    m_ui->actionBottomQueuePos->setIcon(UIThemeManager::instance()->getIcon(u"go-bottom"_s));
+    m_ui->actionDelete->setIcon(UIThemeManager::instance()->getIcon(u"list-remove"_s));
+    m_ui->actionDocumentation->setIcon(UIThemeManager::instance()->getIcon(u"help-contents"_s));
+    m_ui->actionDonateMoney->setIcon(UIThemeManager::instance()->getIcon(u"wallet-open"_s));
+    m_ui->actionExit->setIcon(UIThemeManager::instance()->getIcon(u"application-exit"_s));
+    m_ui->actionLock->setIcon(UIThemeManager::instance()->getIcon(u"object-locked"_s));
+    m_ui->actionOptions->setIcon(UIThemeManager::instance()->getIcon(u"configure"_s, u"preferences-system"_s));
+    m_ui->actionPause->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_s, u"media-playback-pause"_s));
+    m_ui->actionPauseAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-stop"_s, u"media-playback-pause"_s));
+    m_ui->actionStart->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_s, u"media-playback-start"_s));
+    m_ui->actionStartAll->setIcon(UIThemeManager::instance()->getIcon(u"torrent-start"_s, u"media-playback-start"_s));
+    m_ui->menuAutoShutdownOnDownloadsCompletion->setIcon(UIThemeManager::instance()->getIcon(u"task-complete"_s, u"application-exit"_s));
+    m_ui->actionManageCookies->setIcon(UIThemeManager::instance()->getIcon(u"browser-cookies"_s, u"preferences-web-browser-cookies"_s));
+    m_ui->menuLog->setIcon(UIThemeManager::instance()->getIcon(u"help-contents"_s));
+    m_ui->actionCheckForUpdates->setIcon(UIThemeManager::instance()->getIcon(u"view-refresh"_s));
 
     auto *lockMenu = new QMenu(this);
     lockMenu->addAction(tr("&Set Password"), this, &MainWindow::defineUILockPassword);
@@ -191,21 +196,33 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     hSplitter->setChildrenCollapsible(false);
     hSplitter->setFrameShape(QFrame::NoFrame);
 
-    // Name filter
-    m_searchFilter = new LineEdit(this);
-    m_searchFilter->setPlaceholderText(tr("Filter torrent names..."));
-    m_searchFilter->setFixedWidth(200);
-    m_searchFilter->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(m_searchFilter, &QWidget::customContextMenuRequested, this, &MainWindow::showFilterContextMenu);
-    m_searchFilterAction = m_ui->toolBar->insertWidget(m_ui->actionLock, m_searchFilter);
+    // Torrent filter
+    m_columnFilterEdit = new LineEdit;
+    m_columnFilterEdit->setPlaceholderText(tr("Filter torrents..."));
+    m_columnFilterEdit->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    m_columnFilterEdit->setFixedWidth(200);
+    m_columnFilterEdit->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(m_columnFilterEdit, &QWidget::customContextMenuRequested, this, &MainWindow::showFilterContextMenu);
+    auto *columnFilterLabel = new QLabel(tr("Filter by:"));
+    m_columnFilterComboBox = new QComboBox;
+    QHBoxLayout *columnFilterLayout = new QHBoxLayout(m_columnFilterWidget);
+    columnFilterLayout->setContentsMargins(0, 0, 0, 0);
+    auto *columnFilterSpacer = new QWidget(this);
+    columnFilterSpacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+    columnFilterLayout->addWidget(columnFilterSpacer);
+    columnFilterLayout->addWidget(m_columnFilterEdit);
+    columnFilterLayout->addWidget(columnFilterLabel, 0);
+    columnFilterLayout->addWidget(m_columnFilterComboBox, 0);
+    m_columnFilterWidget = new QWidget(this);
+    m_columnFilterWidget->setLayout(columnFilterLayout);
+    m_columnFilterAction = m_ui->toolBar->insertWidget(m_ui->actionLock, m_columnFilterWidget);
 
-    QWidget *spacer = new QWidget(this);
+    auto *spacer = new QWidget(this);
     spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-    m_ui->toolBar->insertWidget(m_searchFilterAction, spacer);
+    m_ui->toolBar->insertWidget(m_columnFilterAction, spacer);
 
     // Transfer List tab
     m_transferListWidget = new TransferListWidget(hSplitter, this);
-    // m_transferListWidget->setStyleSheet("QTreeView {border: none;}");  // borderless
     m_propertiesWidget = new PropertiesWidget(hSplitter);
     connect(m_transferListWidget, &TransferListWidget::currentTorrentChanged, m_propertiesWidget, &PropertiesWidget::loadTorrentInfos);
     hSplitter->addWidget(m_transferListWidget);
@@ -214,11 +231,18 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     m_splitter->setCollapsible(0, false);
     m_tabs->addTab(m_splitter,
 #ifndef Q_OS_MACOS
-        UIThemeManager::instance()->getIcon(u"folder-remote"_qs),
+        UIThemeManager::instance()->getIcon(u"folder-remote"_s),
 #endif
         tr("Transfers"));
-
-    connect(m_searchFilter, &LineEdit::textChanged, m_transferListWidget, &TransferListWidget::applyNameFilter);
+    // Filter types
+    const QVector<TransferListModel::Column> filterTypes = {TransferListModel::Column::TR_NAME, TransferListModel::Column::TR_SAVE_PATH};
+    for (const TransferListModel::Column type : filterTypes)
+    {
+        const QString typeName = m_transferListWidget->getSourceModel()->headerData(type, Qt::Horizontal, Qt::DisplayRole).value<QString>();
+        m_columnFilterComboBox->addItem(typeName, type);
+    }
+    connect(m_columnFilterComboBox, &QComboBox::currentIndexChanged, this, &MainWindow::applyTransferListFilter);
+    connect(m_columnFilterEdit, &LineEdit::textChanged, this, &MainWindow::applyTransferListFilter);
     connect(hSplitter, &QSplitter::splitterMoved, this, &MainWindow::saveSettings);
     connect(m_splitter, &QSplitter::splitterMoved, this, &MainWindow::saveSplitterSettings);
     connect(BitTorrent::Session::instance(), &BitTorrent::Session::trackersChanged, m_propertiesWidget, &PropertiesWidget::loadTrackers);
@@ -278,15 +302,9 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
     connect(m_ui->actionCheckForUpdates, &QAction::triggered, this, [this]() { checkProgramUpdate(true); });
 
-#ifdef Q_OS_WIN
-    if (::IsWindows10OrGreater() && pref->isUpdateCheckEnabled())
-        checkProgramUpdate(false);
-#else
-
     // trigger an early check on startup
     if (pref->isUpdateCheckEnabled())
         checkProgramUpdate(false);
-#endif
 #else
     m_ui->actionCheckForUpdates->setVisible(false);
 #endif
@@ -300,9 +318,11 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
 
     connect(m_ui->actionManageCookies, &QAction::triggered, this, &MainWindow::manageCookies);
 
+    // Initialise system sleep inhibition timer
     m_pwr = new PowerManagement(this);
     m_preventTimer = new QTimer(this);
     connect(m_preventTimer, &QTimer::timeout, this, &MainWindow::updatePowerManagementState);
+    m_preventTimer->start(PREVENT_SUSPEND_INTERVAL);
 
     // Configure BT session according to options
     loadPreferences();
@@ -348,7 +368,7 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     autoShutdownGroup->addAction(m_ui->actionAutoShutdown);
     autoShutdownGroup->addAction(m_ui->actionAutoSuspend);
     autoShutdownGroup->addAction(m_ui->actionAutoHibernate);
-#if (!defined(Q_OS_UNIX) || defined(Q_OS_MACOS)) || defined(QT_DBUS_LIB)
+#if (!defined(Q_OS_UNIX) || defined(Q_OS_MACOS)) || defined(QBT_USES_DBUS)
     m_ui->actionAutoShutdown->setChecked(pref->shutdownWhenDownloadsComplete());
     m_ui->actionAutoSuspend->setChecked(pref->suspendWhenDownloadsComplete());
     m_ui->actionAutoHibernate->setChecked(pref->hibernateWhenDownloadsComplete());
@@ -385,27 +405,27 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     });
 
 #ifdef Q_OS_MACOS
-    // Make sure the Window is visible if we don't have a tray icon
-    if (initialState == Minimized)
-    {
-        showMinimized();
-    }
-    else
+    if (initialState == WindowState::Normal)
     {
         show();
         activateWindow();
         raise();
     }
+    else
+    {
+        // Make sure the Window is visible if we don't have a tray icon
+        showMinimized();
+    }
 #else
     if (app->desktopIntegration()->isActive())
     {
-        if ((initialState != Minimized) && !m_uiLocked)
+        if ((initialState == WindowState::Normal) && !m_uiLocked)
         {
             show();
             activateWindow();
             raise();
         }
-        else if (initialState == Minimized)
+        else if (initialState == WindowState::Minimized)
         {
             showMinimized();
             if (pref->minimizeToTray())
@@ -422,7 +442,7 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     else
     {
         // Make sure the Window is visible if we don't have a tray icon
-        if (initialState == Minimized)
+        if (initialState != WindowState::Normal)
         {
             showMinimized();
         }
@@ -435,8 +455,6 @@ MainWindow::MainWindow(IGUIApplication *app, const State initialState)
     }
 #endif
 
-    m_propertiesWidget->readSettings();
-
     const bool isFiltersSidebarVisible = pref->isFiltersSidebarVisible();
     m_ui->actionShowFiltersSidebar->setChecked(isFiltersSidebarVisible);
     if (isFiltersSidebarVisible)
@@ -658,7 +676,7 @@ void MainWindow::displayRSSTab(bool enable)
             m_tabs->addTab(m_rssWidget, tr("RSS (%1)").arg(RSS::Session::instance()->rootFolder()->unreadCount()));
 #else
             const int indexTab = m_tabs->addTab(m_rssWidget, tr("RSS (%1)").arg(RSS::Session::instance()->rootFolder()->unreadCount()));
-            m_tabs->setTabIcon(indexTab, UIThemeManager::instance()->getIcon(u"application-rss"_qs));
+            m_tabs->setTabIcon(indexTab, UIThemeManager::instance()->getIcon(u"application-rss"_s));
 #endif
         }
     }
@@ -672,7 +690,7 @@ void MainWindow::showFilterContextMenu()
 {
     const Preferences *pref = Preferences::instance();
 
-    QMenu *menu = m_searchFilter->createStandardContextMenu();
+    QMenu *menu = m_columnFilterEdit->createStandardContextMenu();
     menu->setAttribute(Qt::WA_DeleteOnClose);
     menu->addSeparator();
 
@@ -680,7 +698,7 @@ void MainWindow::showFilterContextMenu()
     useRegexAct->setCheckable(true);
     useRegexAct->setChecked(pref->getRegexAsFilteringPatternForTransferList());
     connect(useRegexAct, &QAction::toggled, pref, &Preferences::setRegexAsFilteringPatternForTransferList);
-    connect(useRegexAct, &QAction::toggled, this, [this]() { m_transferListWidget->applyNameFilter(m_searchFilter->text()); });
+    connect(useRegexAct, &QAction::toggled, this, &MainWindow::applyTransferListFilter);
 
     menu->popup(QCursor::pos());
 }
@@ -696,7 +714,7 @@ void MainWindow::displaySearchTab(bool enable)
             m_searchWidget = new SearchWidget(app(), this);
             m_tabs->insertTab(1, m_searchWidget,
 #ifndef Q_OS_MACOS
-                UIThemeManager::instance()->getIcon(u"edit-find"_qs),
+                UIThemeManager::instance()->getIcon(u"edit-find"_s),
 #endif
                 tr("Search"));
         }
@@ -709,8 +727,8 @@ void MainWindow::displaySearchTab(bool enable)
 
 void MainWindow::focusSearchFilter()
 {
-    m_searchFilter->setFocus();
-    m_searchFilter->selectAll();
+    m_columnFilterEdit->setFocus();
+    m_columnFilterEdit->selectAll();
 }
 
 void MainWindow::updateNbTorrents()
@@ -720,7 +738,7 @@ void MainWindow::updateNbTorrents()
 
 void MainWindow::on_actionDocumentation_triggered() const
 {
-    QDesktopServices::openUrl(QUrl(u"http://doc.qbittorrent.org"_qs));
+    QDesktopServices::openUrl(QUrl(u"https://doc.qbittorrent.org"_s));
 }
 
 void MainWindow::tabChanged(int newTab)
@@ -732,10 +750,10 @@ void MainWindow::tabChanged(int newTab)
     {
         qDebug("Changed tab to transfer list, refreshing the list");
         m_propertiesWidget->loadDynamicData();
-        m_searchFilterAction->setVisible(true);
+        m_columnFilterAction->setVisible(true);
         return;
     }
-    m_searchFilterAction->setVisible(false);
+    m_columnFilterAction->setVisible(false);
 
     if (m_tabs->currentWidget() == m_searchWidget)
     {
@@ -837,6 +855,8 @@ void MainWindow::createKeyboardShortcuts()
     connect(switchExecutionLogShortcut, &QShortcut::activated, this, &MainWindow::displayExecutionLogTab);
     const auto *switchSearchFilterShortcut = new QShortcut(QKeySequence::Find, m_transferListWidget);
     connect(switchSearchFilterShortcut, &QShortcut::activated, this, &MainWindow::focusSearchFilter);
+    const auto *switchSearchFilterShortcutAlternative = new QShortcut((Qt::CTRL | Qt::Key_E), m_transferListWidget);
+    connect(switchSearchFilterShortcutAlternative, &QShortcut::activated, this, &MainWindow::focusSearchFilter);
 
     m_ui->actionDocumentation->setShortcut(QKeySequence::HelpContents);
     m_ui->actionOptions->setShortcut(Qt::ALT | Qt::Key_O);
@@ -929,7 +949,7 @@ void MainWindow::askRecursiveTorrentDownloadConfirmation(const BitTorrent::Torre
 
 void MainWindow::on_actionSetGlobalSpeedLimits_triggered()
 {
-    auto dialog = new SpeedLimitDialog {this};
+    auto *dialog = new SpeedLimitDialog {this};
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->open();
 }
@@ -1070,6 +1090,12 @@ void MainWindow::showEvent(QShowEvent *e)
     {
         // preparations before showing the window
 
+        if (m_neverShown)
+        {
+            m_propertiesWidget->readSettings();
+            m_neverShown = false;
+        }
+
         if (currentTabWidget() == m_transferListWidget)
             m_propertiesWidget->loadDynamicData();
 
@@ -1156,7 +1182,7 @@ void MainWindow::closeEvent(QCloseEvent *e)
             if (!isVisible())
                 show();
             QMessageBox confirmBox(QMessageBox::Question, tr("Exiting qBittorrent"),
-                                   // Split it because the last sentence is used in the Web UI
+                                   // Split it because the last sentence is used in the WebUI
                                    tr("Some files are currently transferring.") + u'\n' + tr("Are you sure you want to quit qBittorrent?"),
                                    QMessageBox::NoButton, this);
             QPushButton *noBtn = confirmBox.addButton(tr("&No"), QMessageBox::NoRole);
@@ -1318,7 +1344,7 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event)
     for (const QString &mime : asConst(event->mimeData()->formats()))
         qDebug("mimeData: %s", mime.toLocal8Bit().data());
 
-    if (event->mimeData()->hasFormat(u"text/plain"_qs) || event->mimeData()->hasFormat(u"text/uri-list"_qs))
+    if (event->mimeData()->hasFormat(u"text/plain"_s) || event->mimeData()->hasFormat(u"text/uri-list"_s))
         event->acceptProposedAction();
 }
 
@@ -1431,25 +1457,13 @@ void MainWindow::loadPreferences()
     else
     {
         // Clear search filter before hiding the top toolbar
-        m_searchFilter->clear();
+        m_columnFilterEdit->clear();
         m_ui->toolBar->setVisible(false);
     }
 
     showStatusBar(pref->isStatusbarDisplayed());
 
-    if (pref->preventFromSuspendWhenDownloading() || pref->preventFromSuspendWhenSeeding())
-    {
-        if (!m_preventTimer->isActive())
-        {
-            updatePowerManagementState();
-            m_preventTimer->start(PREVENT_SUSPEND_INTERVAL);
-        }
-    }
-    else
-    {
-        m_preventTimer->stop();
-        m_pwr->setActivityState(false);
-    }
+    updatePowerManagementState();
 
     m_transferListWidget->setAlternatingRowColors(pref->useAlternatingRowColors());
     m_propertiesWidget->getFilesList()->setAlternatingRowColors(pref->useAlternatingRowColors());
@@ -1494,11 +1508,7 @@ void MainWindow::loadPreferences()
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
     if (pref->isUpdateCheckEnabled())
     {
-#ifdef Q_OS_WIN
-        if (::IsWindows10OrGreater() && !m_programUpdateTimer)
-#else
         if (!m_programUpdateTimer)
-#endif
         {
             m_programUpdateTimer = new QTimer(this);
             m_programUpdateTimer->setInterval(24h);
@@ -1533,7 +1543,7 @@ void MainWindow::reloadSessionStats()
         MacUtils::setBadgeLabelText({});
     }
 #else
-    const auto toolTip = u"%1\n%2"_qs.arg(
+    const auto toolTip = u"%1\n%2"_s.arg(
         tr("DL speed: %1", "e.g: Download speed: 10 KiB/s").arg(Utils::Misc::friendlyUnit(status.payloadDownloadRate, true))
         , tr("UP speed: %1", "e.g: Upload speed: 10 KiB/s").arg(Utils::Misc::friendlyUnit(status.payloadUploadRate, true)));
     app()->desktopIntegration()->setToolTip(toolTip); // tray icon
@@ -1713,14 +1723,14 @@ void MainWindow::on_actionSearchWidget_triggered()
 #ifdef Q_OS_WIN
             const QMessageBox::StandardButton buttonPressed = QMessageBox::question(this, tr("Old Python Runtime")
                 , tr("Your Python version (%1) is outdated. Minimum requirement: %2.\nDo you want to install a newer version now?")
-                    .arg(pyInfo.version.toString(), u"3.5.0")
+                    .arg(pyInfo.version.toString(), u"3.7.0")
                 , (QMessageBox::Yes | QMessageBox::No), QMessageBox::Yes);
             if (buttonPressed == QMessageBox::Yes)
                 installPython();
 #else
             QMessageBox::information(this, tr("Old Python Runtime")
                 , tr("Your Python version (%1) is outdated. Please upgrade to latest version for search engines to work.\nMinimum requirement: %2.")
-                .arg(pyInfo.version.toString(), u"3.5.0"));
+                .arg(pyInfo.version.toString(), u"3.7.0"));
 #endif
             return;
         }
@@ -1771,7 +1781,7 @@ void MainWindow::handleUpdateCheckFinished(ProgramUpdater *updater, const bool i
     {
         const QString msg {tr("A new version is available.") + u"<br/>"
             + tr("Do you want to download %1?").arg(newVersion) + u"<br/><br/>"
-            + u"<a href=\"https://www.qbittorrent.org/news.php\">%1</a>"_qs.arg(tr("Open changelog..."))};
+            + u"<a href=\"https://www.qbittorrent.org/news.php\">%1</a>"_s.arg(tr("Open changelog..."))};
         auto *msgBox = new QMessageBox {QMessageBox::Question, tr("qBittorrent Update Available"), msg
             , (QMessageBox::Yes | QMessageBox::No), this};
         msgBox->setAttribute(Qt::WA_DeleteOnClose);
@@ -1792,7 +1802,7 @@ void MainWindow::handleUpdateCheckFinished(ProgramUpdater *updater, const bool i
     {
         if (invokedByUser)
         {
-            auto *msgBox = new QMessageBox {QMessageBox::Information, u"qBittorrent"_qs
+            auto *msgBox = new QMessageBox {QMessageBox::Information, u"qBittorrent"_s
                 , tr("No updates available.\nYou are already using the latest version.")
                 , QMessageBox::Ok, this};
             msgBox->setAttribute(Qt::WA_DeleteOnClose);
@@ -1816,7 +1826,7 @@ void MainWindow::toggleAlternativeSpeeds()
 
 void MainWindow::on_actionDonateMoney_triggered()
 {
-    QDesktopServices::openUrl(QUrl(u"https://www.qbittorrent.org/donate"_qs));
+    QDesktopServices::openUrl(QUrl(u"https://www.qbittorrent.org/donate"_s));
 }
 
 void MainWindow::showConnectionSettings()
@@ -1840,7 +1850,7 @@ void MainWindow::on_actionExecutionLogs_triggered(bool checked)
         m_tabs->addTab(m_executionLog, tr("Execution Log"));
 #else
         const int indexTab = m_tabs->addTab(m_executionLog, tr("Execution Log"));
-        m_tabs->setTabIcon(indexTab, UIThemeManager::instance()->getIcon(u"help-contents"_qs));
+        m_tabs->setTabIcon(indexTab, UIThemeManager::instance()->getIcon(u"help-contents"_s));
 #endif
     }
     else
@@ -1917,38 +1927,31 @@ void MainWindow::on_actionAutoShutdown_toggled(bool enabled)
 
 void MainWindow::updatePowerManagementState()
 {
+    const bool preventFromSuspendWhenDownloading = Preferences::instance()->preventFromSuspendWhenDownloading();
+    const bool preventFromSuspendWhenSeeding = Preferences::instance()->preventFromSuspendWhenSeeding();
+
     const QVector<BitTorrent::Torrent *> allTorrents = BitTorrent::Session::instance()->torrents();
-    const bool hasUnfinishedTorrents = std::any_of(allTorrents.cbegin(), allTorrents.cend(), [](const BitTorrent::Torrent *torrent)
+    const bool inhibitSuspend = std::any_of(allTorrents.cbegin(), allTorrents.cend(), [&](const BitTorrent::Torrent *torrent)
     {
-        return (!torrent->isSeed() && !torrent->isPaused() && !torrent->isErrored() && torrent->hasMetadata());
+        if (preventFromSuspendWhenDownloading && (!torrent->isFinished() && !torrent->isPaused() && !torrent->isErrored() && torrent->hasMetadata()))
+            return true;
+
+        if (preventFromSuspendWhenSeeding && (torrent->isFinished() && !torrent->isPaused()))
+            return true;
+
+        return torrent->isMoving();
     });
-    const bool hasRunningSeed = std::any_of(allTorrents.cbegin(), allTorrents.cend(), [](const BitTorrent::Torrent *torrent)
-    {
-        return (torrent->isSeed() && !torrent->isPaused());
-    });
-    const bool inhibitSuspend = (Preferences::instance()->preventFromSuspendWhenDownloading() && hasUnfinishedTorrents)
-                             || (Preferences::instance()->preventFromSuspendWhenSeeding() && hasRunningSeed);
     m_pwr->setActivityState(inhibitSuspend);
 }
 
+void MainWindow::applyTransferListFilter()
+{
+    m_transferListWidget->applyFilter(m_columnFilterEdit->text(), m_columnFilterComboBox->currentData().value<TransferListModel::Column>());
+}
+
 #if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
 void MainWindow::checkProgramUpdate(const bool invokedByUser)
 {
-#ifdef Q_OS_WIN
-    if (!::IsWindows10OrGreater())
-    {
-        if (invokedByUser) {
-            auto *msgBox = new QMessageBox {QMessageBox::Information, u"qBittorrent"_qs
-                                           , tr("This is the last supported version for your Windows version.")
-                                           , QMessageBox::Ok, this};
-            msgBox->setAttribute(Qt::WA_DeleteOnClose);
-            msgBox->show();
-        }
-
-        return;
-    }
-#endif
-
     if (m_programUpdateTimer)
         m_programUpdateTimer->stop();
 
@@ -1972,13 +1975,18 @@ void MainWindow::installPython()
     setCursor(QCursor(Qt::WaitCursor));
     // Download python
 #ifdef QBT_APP_64BIT
-    const auto installerURL = u"https://www.python.org/ftp/python/3.8.10/python-3.8.10-amd64.exe"_qs;
+    const auto installerURL = ::IsWindows8OrGreater()
+        ? u"https://www.python.org/ftp/python/3.10.11/python-3.10.11-amd64.exe"_s
+        : u"https://www.python.org/ftp/python/3.8.10/python-3.8.10-amd64.exe"_s;
 #else
-    const auto installerURL = u"https://www.python.org/ftp/python/3.8.10/python-3.8.10.exe"_qs;
+    const auto installerURL = ::IsWindows8OrGreater()
+        ? u"https://www.python.org/ftp/python/3.10.11/python-3.10.11.exe"_s
+        : u"https://www.python.org/ftp/python/3.8.10/python-3.8.10.exe"_s;
 #endif
     Net::DownloadManager::instance()->download(
-                Net::DownloadRequest(installerURL).saveToFile(true)
-                , this, &MainWindow::pythonDownloadFinished);
+            Net::DownloadRequest(installerURL).saveToFile(true)
+            , Preferences::instance()->useProxyForGeneralPurposes()
+            , this, &MainWindow::pythonDownloadFinished);
 }
 
 void MainWindow::pythonDownloadFinished(const Net::DownloadResult &result)
@@ -1999,7 +2007,7 @@ void MainWindow::pythonDownloadFinished(const Net::DownloadResult &result)
 
     const Path exePath = result.filePath + u".exe";
     Utils::Fs::renameFile(result.filePath, exePath);
-    installer.start(exePath.toString(), {u"/passive"_qs});
+    installer.start(exePath.toString(), {u"/passive"_s});
 
     // Wait for setup to complete
     installer.waitForFinished(10 * 60 * 1000);
