@@ -33,8 +33,10 @@ let alternativeSpeedLimits = false;
 let queueing_enabled = true;
 let serverSyncMainDataInterval = 1500;
 let customSyncMainDataInterval = null;
+let useSubcategories = true;
 let searchTabInitialized = false;
 let rssTabInitialized = false;
+let logTabInitialized = false;
 
 let syncRequestInProgress = false;
 
@@ -93,6 +95,32 @@ const getShowFiltersSidebar = function() {
     return (show === null) || (show === 'true');
 };
 
+// getHost emulate the GUI version `QString getHost(const QString &url)`
+function getHost(url) {
+    // We want the hostname.
+    // If failed to parse the domain, original input should be returned
+
+    if (!/^(?:https?|udp):/i.test(url)) {
+        return url;
+    }
+
+    try {
+        // hack: URL can not get hostname from udp protocol
+        const parsedUrl = new URL(url.replace(/^udp:/i, 'https:'));
+        // host: "example.com:8443"
+        // hostname: "example.com"
+        const host = parsedUrl.hostname;
+        if (!host) {
+            return url;
+        }
+
+        return host;
+    }
+    catch (error) {
+        return url;
+    }
+}
+
 function genHash(string) {
     // origins:
     // https://stackoverflow.com/a/8831937
@@ -190,9 +218,21 @@ window.addEvent('load', function() {
         $("rssTabColumn").addClass("invisible");
     };
 
+    const buildLogTab = function() {
+        new MochaUI.Column({
+            id: 'logTabColumn',
+            placement: 'main',
+            width: null
+        });
+
+        // start off hidden
+        $('logTabColumn').addClass('invisible');
+    };
+
     buildTransfersTab();
     buildSearchTab();
     buildRssTab();
+    buildLogTab();
     MochaUI.initializeTabs('mainWindowTabsList');
 
     setCategoryFilter = function(hash) {
@@ -304,6 +344,7 @@ window.addEvent('load', function() {
     // After showing/hiding the toolbar + status bar
     let showSearchEngine = LocalPreferences.get('show_search_engine') !== "false";
     let showRssReader = LocalPreferences.get('show_rss_reader') !== "false";
+    let showLogViewer = LocalPreferences.get('show_log_viewer') === 'true';
 
     // After Show Top Toolbar
     MochaUI.Desktop.setDesktopSize();
@@ -415,9 +456,17 @@ window.addEvent('load', function() {
         categoryList.empty();
 
         const create_link = function(hash, text, count) {
-            const html = '<a href="#" onclick="setCategoryFilter(' + hash + ');return false;">'
+            let display_name = text;
+            let margin_left = 0;
+            if (useSubcategories) {
+                const category_path = text.split("/");
+                display_name = category_path[category_path.length - 1];
+                margin_left = (category_path.length - 1) * 20;
+            }
+
+            const html = '<a href="#" style="margin-left: ' + margin_left + 'px" onclick="setCategoryFilter(' + hash + ');return false;">'
                 + '<img src="images/view-categories.svg"/>'
-                + window.qBittorrent.Misc.escapeHtml(text) + ' (' + count + ')' + '</a>';
+                + window.qBittorrent.Misc.escapeHtml(display_name) + ' (' + count + ')' + '</a>';
             const el = new Element('li', {
                 id: hash,
                 html: html
@@ -439,14 +488,37 @@ window.addEvent('load', function() {
         Object.each(category_list, function(category) {
             sortedCategories.push(category.name);
         });
-        sortedCategories.sort();
+        sortedCategories.sort(function(category1, category2) {
+            for (let i = 0; i < Math.min(category1.length, category2.length); ++i) {
+                if (category1[i] === "/" && category2[i] !== "/") {
+                    return -1;
+                }
+                else if (category1[i] !== "/" && category2[i] === "/") {
+                    return 1;
+                }
+                else if (category1[i] !== category2[i]) {
+                    return category1[i].localeCompare(category2[i]);
+                }
+            }
 
-        Object.each(sortedCategories, function(categoryName) {
-            const categoryHash = genHash(categoryName);
-            const categoryCount = category_list[categoryHash].torrents.length;
-            categoryList.appendChild(create_link(categoryHash, categoryName, categoryCount));
+            return category1.length - category2.length;
         });
 
+        for (let i = 0; i < sortedCategories.length; ++i) {
+            const categoryName = sortedCategories[i];
+            const categoryHash = genHash(categoryName);
+            let categoryCount = category_list[categoryHash].torrents.length;
+
+            if (useSubcategories) {
+                for (let j = i + 1; j < sortedCategories.length && sortedCategories[j].startsWith(categoryName + "/"); ++j) {
+                    const hash = genHash(sortedCategories[j]);
+                    categoryCount += category_list[hash].torrents.length;
+                }
+            }
+
+            categoryList.appendChild(create_link(categoryHash, categoryName, categoryCount));
+        }
+
         highlightSelectedCategory();
     };
 
@@ -546,8 +618,15 @@ window.addEvent('load', function() {
         }
         trackerFilterList.appendChild(createLink(TRACKERS_TRACKERLESS, 'QBT_TR(Trackerless (%1))QBT_TR[CONTEXT=TrackerFiltersList]', trackerlessTorrentsCount));
 
-        for (const [hash, tracker] of trackerList)
-            trackerFilterList.appendChild(createLink(hash, tracker.url + ' (%1)', tracker.torrents.length));
+        // Sort trackers by hostname
+        const sortedList = [...trackerList.entries()].sort((left, right) => {
+            const leftHost = getHost(left[1].url);
+            const rightHost = getHost(right[1].url);
+            return window.qBittorrent.Misc.naturalSortCollator.compare(leftHost, rightHost);
+        });
+        for (const [hash, tracker] of sortedList) {
+            trackerFilterList.appendChild(createLink(hash, (getHost(tracker.url) + ' (%1)'), tracker.torrents.length));
+        }
 
         highlightSelectedTracker();
     };
@@ -642,10 +721,29 @@ window.addEvent('load', function() {
                     if (response['trackers']) {
                         for (const tracker in response['trackers']) {
                             const torrents = response['trackers'][tracker];
-                            const hash = genHash(tracker);
+                            const hash = genHash(getHost(tracker));
+
+                            // the reason why we need the merge here is because the web ui api returned trackers may have different url for the same tracker host.
+                            // for example, some private trackers use diff urls for each torrent from the same tracker host.
+                            // then we got the response of `trackers` from qBittorrent api will like:
+                            // {
+                            //     "trackers": {
+                            //         "https://example.com/announce?passkey=identify_info1": ["hash1"],
+                            //         "https://example.com/announce?passkey=identify_info2": ["hash2"],
+                            //         "https://example.com/announce?passkey=identify_info3": ["hash3"]
+                            //     }
+                            // }
+                            // after getHost(), those torrents all belongs to `example.com`
+                            let merged_torrents = torrents;
+                            if (trackerList.has(hash)) {
+                                merged_torrents = trackerList.get(hash).torrents.concat(torrents);
+                                // deduplicate is needed when the webui opens in multi tabs
+                                merged_torrents = merged_torrents.filter((item, pos) => merged_torrents.indexOf(item) === pos);
+                            }
+
                             trackerList.set(hash, {
                                 url: tracker,
-                                torrents: torrents
+                                torrents: merged_torrents
                             });
                         }
                         updateTrackers = true;
@@ -653,7 +751,7 @@ window.addEvent('load', function() {
                     if (response['trackers_removed']) {
                         for (let i = 0; i < response['trackers_removed'].length; ++i) {
                             const tracker = response['trackers_removed'][i];
-                            const hash = genHash(tracker);
+                            const hash = genHash(getHost(tracker));
                             trackerList.delete(hash);
                         }
                         updateTrackers = true;
@@ -769,14 +867,17 @@ window.addEvent('load', function() {
             case 'connected':
                 $('connectionStatus').src = 'images/connected.svg';
                 $('connectionStatus').alt = 'QBT_TR(Connection status: Connected)QBT_TR[CONTEXT=MainWindow]';
+                $('connectionStatus').title = 'QBT_TR(Connection status: Connected)QBT_TR[CONTEXT=MainWindow]';
                 break;
             case 'firewalled':
                 $('connectionStatus').src = 'images/firewalled.svg';
                 $('connectionStatus').alt = 'QBT_TR(Connection status: Firewalled)QBT_TR[CONTEXT=MainWindow]';
+                $('connectionStatus').title = 'QBT_TR(Connection status: Firewalled)QBT_TR[CONTEXT=MainWindow]';
                 break;
             default:
                 $('connectionStatus').src = 'images/disconnected.svg';
                 $('connectionStatus').alt = 'QBT_TR(Connection status: Disconnected)QBT_TR[CONTEXT=MainWindow]';
+                $('connectionStatus').title = 'QBT_TR(Connection status: Disconnected)QBT_TR[CONTEXT=MainWindow]';
                 break;
         }
 
@@ -807,6 +908,11 @@ window.addEvent('load', function() {
             updateAltSpeedIcon(alternativeSpeedLimits);
         }
 
+        if (useSubcategories != serverState.use_subcategories) {
+            useSubcategories = serverState.use_subcategories;
+            updateCategoryList();
+        }
+
         serverSyncMainDataInterval = Math.max(serverState.refresh_interval, 500);
     };
 
@@ -814,10 +920,12 @@ window.addEvent('load', function() {
         if (enabled) {
             $('alternativeSpeedLimits').src = 'images/slow.svg';
             $('alternativeSpeedLimits').alt = 'QBT_TR(Alternative speed limits: On)QBT_TR[CONTEXT=MainWindow]';
+            $('alternativeSpeedLimits').title = 'QBT_TR(Alternative speed limits: On)QBT_TR[CONTEXT=MainWindow]';
         }
         else {
             $('alternativeSpeedLimits').src = 'images/slow_off.svg';
             $('alternativeSpeedLimits').alt = 'QBT_TR(Alternative speed limits: Off)QBT_TR[CONTEXT=MainWindow]';
+            $('alternativeSpeedLimits').title = 'QBT_TR(Alternative speed limits: Off)QBT_TR[CONTEXT=MainWindow]';
         }
     };
 
@@ -912,6 +1020,12 @@ window.addEvent('load', function() {
         updateTabDisplay();
     });
 
+    $('showLogViewerLink').addEvent('click', function(e) {
+        showLogViewer = !showLogViewer;
+        LocalPreferences.set('show_log_viewer', showLogViewer.toString());
+        updateTabDisplay();
+    });
+
     const updateTabDisplay = function() {
         if (showRssReader) {
             $('showRssReaderLink').firstChild.style.opacity = '1';
@@ -941,8 +1055,22 @@ window.addEvent('load', function() {
                 $("transfersTabLink").click();
         }
 
+        if (showLogViewer) {
+            $('showLogViewerLink').firstChild.style.opacity = '1';
+            $('mainWindowTabs').removeClass('invisible');
+            $('logTabLink').removeClass('invisible');
+            if (!MochaUI.Panels.instances.LogPanel)
+                addLogPanel();
+        }
+        else {
+            $('showLogViewerLink').firstChild.style.opacity = '0';
+            $('logTabLink').addClass('invisible');
+            if ($('logTabLink').hasClass('selected'))
+                $("transfersTabLink").click();
+        }
+
         // display no tabs
-        if (!showRssReader && !showSearchEngine)
+        if (!showRssReader && !showSearchEngine && !showLogViewer)
             $('mainWindowTabs').addClass('invisible');
     };
 
@@ -954,18 +1082,21 @@ window.addEvent('load', function() {
         $("filtersColumn").removeClass("invisible");
         $("filtersColumn_handle").removeClass("invisible");
         $("mainColumn").removeClass("invisible");
+        $('torrentsFilterToolbar').removeClass("invisible");
 
         customSyncMainDataInterval = null;
         syncData(100);
 
         hideSearchTab();
         hideRssTab();
+        hideLogTab();
     };
 
     const hideTransfersTab = function() {
         $("filtersColumn").addClass("invisible");
         $("filtersColumn_handle").addClass("invisible");
         $("mainColumn").addClass("invisible");
+        $('torrentsFilterToolbar').addClass("invisible");
         MochaUI.Desktop.resizePanels();
     };
 
@@ -979,6 +1110,7 @@ window.addEvent('load', function() {
         customSyncMainDataInterval = 30000;
         hideTransfersTab();
         hideRssTab();
+        hideLogTab();
     };
 
     const hideSearchTab = function() {
@@ -999,14 +1131,37 @@ window.addEvent('load', function() {
         customSyncMainDataInterval = 30000;
         hideTransfersTab();
         hideSearchTab();
+        hideLogTab();
     };
 
     const hideRssTab = function() {
         $("rssTabColumn").addClass("invisible");
-        window.qBittorrent.Rss.unload();
+        window.qBittorrent.Rss && window.qBittorrent.Rss.unload();
         MochaUI.Desktop.resizePanels();
     };
 
+    const showLogTab = function() {
+        if (!logTabInitialized) {
+            window.qBittorrent.Log.init();
+            logTabInitialized = true;
+        }
+        else {
+            window.qBittorrent.Log.load();
+        }
+
+        $('logTabColumn').removeClass('invisible');
+        customSyncMainDataInterval = 30000;
+        hideTransfersTab();
+        hideSearchTab();
+        hideRssTab();
+    };
+
+    const hideLogTab = function() {
+        $('logTabColumn').addClass('invisible');
+        MochaUI.Desktop.resizePanels();
+        window.qBittorrent.Log && window.qBittorrent.Log.unload();
+    };
+
     const addSearchPanel = function() {
         new MochaUI.Panel({
             id: 'SearchPanel',
@@ -1045,6 +1200,42 @@ window.addEvent('load', function() {
         });
     };
 
+    var addLogPanel = function() {
+        new MochaUI.Panel({
+            id: 'LogPanel',
+            title: 'Log',
+            header: true,
+            padding: {
+                top: 0,
+                right: 0,
+                bottom: 0,
+                left: 0
+            },
+            loadMethod: 'xhr',
+            contentURL: 'views/log.html',
+            require: {
+                css: ['css/lib/vanillaSelectBox.css'],
+                js: ['scripts/lib/vanillaSelectBox.js'],
+            },
+            tabsURL: 'views/logTabs.html',
+            tabsOnload: function() {
+                MochaUI.initializeTabs('panelTabs');
+
+                $('logMessageLink').addEvent('click', function(e) {
+                    window.qBittorrent.Log.setCurrentTab('main');
+                });
+
+                $('logPeerLink').addEvent('click', function(e) {
+                    window.qBittorrent.Log.setCurrentTab('peer');
+                });
+            },
+            collapsible: false,
+            content: '',
+            column: 'logTabColumn',
+            height: null
+        });
+    };
+
     new MochaUI.Panel({
         id: 'transferList',
         title: 'Panel',
@@ -1185,6 +1376,7 @@ window.addEvent('load', function() {
     $('transfersTabLink').addEvent('click', showTransfersTab);
     $('searchTabLink').addEvent('click', showSearchTab);
     $('rssTabLink').addEvent('click', showRssTab);
+    $('logTabLink').addEvent('click', showLogTab);
     updateTabDisplay();
 
     const registerDragAndDrop = () => {
